//  Copyright 2019 Kakao Corp.
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.

import UIKit

import SafariServices
import Alamofire

import KakaoSDKCommon
import KakaoSDKAuth
import KakaoSDKUser
import KakaoSDKTalk
import KakaoSDKStory
import KakaoSDKLink
import KakaoSDKNavi
import KakaoSDKTemplate

import KakaoPartnerSDKCommon
import KakaoPartnerSDKUser
import KakaoPartnerSDKTalk
import KakaoPartnerSDKAuth

extension PartnerSDKSampleViewController {
    private func updateResultIndicator(cell:UITableViewCell?, row:Row) {
        
        switch(row.apiResult) {
        case .Ready: cell?.backgroundColor = UIColor.white
        case .Success: cell?.backgroundColor = UIColor.green
        case .Fail: cell?.backgroundColor = UIColor.red
        }
    }
    
    private func updateResultIndicator() {
        if let resultIndicator = self.resultIndicator {
            let cell = self.tableView.cellForRow(at: resultIndicator.0)
            self.updateResultIndicator(cell: cell, row: resultIndicator.1)
        }
    }
    
    private func success(_ object: Any? = nil, updateIndicator: Bool = true) {
        if let object = object {
            print("\(String(describing:object))")
        }
        
        if (updateIndicator) {
            self.resultIndicator?.1.apiResult = .Success
            self.updateResultIndicator()
        }
    }
    
    private func errorHandler(error:Error?, updateIndicator: Bool = true) {
        guard error != nil else { return }
        if (updateIndicator) {
            self.resultIndicator?.1.apiResult = .Fail
            self.updateResultIndicator()
        }
        
        if let sdkError = error as? SdkError {
            print("[sdk error handling start]------------------------------------------------------------------------------")
            
            switch(sdkError) {
            case .ApiFailed:
                let apiError = sdkError.getApiError()
                switch(apiError.reason) {
                default:
                    print("ApiError 서드 개발자 에러처리 필요.")
                }
                break
            case .AuthFailed:
                let authError = sdkError.getAuthError()
                switch(authError.reason) {
                default:
                    print("AuthError 서드 개발자 에러처리 필요.")
                }
                break
            case .ClientFailed:
                break
            case .GeneralFailed:
                break
            }
            print("[sdk error handling end]--------------------------------------------------------------------------------\n")
        }
        else {
            print("[not sdk error]------------------------------------------------------------------------------")
            print("\(String(describing: error))")
        }
    }
}


public class PartnerSDKSampleViewController: SDKSampleBaseViewController, UITableViewDelegate, UITableViewDataSource {
    
    var safariViewController : SFSafariViewController?
    var authenticationSession : SFAuthenticationSession?
    
    var recursiveAppFriendsCompletion : ((Friends<PartnerFriend>?, Error?) -> Void)!
    
    
    //for ui
    var resultIndicator : (IndexPath, Row)?
    
    override public func loadView() {
        super.loadView()
        
        setupViews()
    }
    
    override public func viewDidLoad() {
        super.viewDidLoad()
        
        self.tableView.delegate = self
        self.tableView.dataSource = self
        
        setMenu()
    }
    
    override public func setupViews() {
        super.setupViews()
        self.tableView.register(UITableViewCell.self, forCellReuseIdentifier: "Cell")
    }
    
    public func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        let item = self.sections![indexPath.section].items[indexPath.row]
        if let action = item.action {
            action()
            
            self.resultIndicator = (indexPath, item)
            
            DispatchQueue.main.asyncAfter(deadline: .now() + 0.1 * 1) {
               tableView.deselectRow(at: indexPath, animated: false)
            }
        }
    }
    
    public func numberOfSections(in tableView: UITableView) -> Int {
        if let sections = self.sections, sections.count != 0 {
            print("sections count: ", sections.count)
            return sections.count
        }
        else {
            return 0
        }
    }
    
    public func tableView(_ tableView: UITableView, titleForHeaderInSection section: Int) -> String? {
        if let sections = self.sections, sections.count != 0 {
            return sections[section].header
        }
        else {
            return nil
        }
    }
    
    public func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        if let sections = self.sections, sections.count != 0 {
            return sections[section].items.count
        }
        else {
            return 0
        }
    }

    public func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "Cell", for: indexPath)
        
        let item = self.sections![indexPath.section].items[indexPath.row]
        
        self.updateResultIndicator(cell:cell, row:item)
        cell.textLabel?.adjustsFontSizeToFitWidth = true
        cell.textLabel?.text = "\(item.title)"
        cell.textLabel?.backgroundColor = UIColor.init(white: 1.0, alpha: 0)
        cell.textLabel?.textColor = UIColor.black
        return cell
    }
    
    public func setTitle(title:String = "SDK Sample", args: String? = nil) {
        self.navigationItem.title = (args != nil) ? "\(title)(\(args!))" : "\(title)"
    }
    
    func extractPickerElements(friends:Friends<PartnerFriend>, receiverIdType:TalkApi.ReceiverIdType = .Uuid) -> [(URL?, String?, Any)]? {
        return friends.elements?.map({ (friend) -> (URL?, String?, Any) in
            
            switch (receiverIdType) {
            case .UserId:
                return (friend.profileThumbnailImage, friend.profileNickname, "\(friend.id ?? 0)")
            case .Uuid:
                return (friend.profileThumbnailImage, friend.profileNickname, friend.uuid)
            default:
                return (friend.profileThumbnailImage, friend.profileNickname, friend.uuid)
            }
        })
    }
    
    func extractPickerElements(chats:Chats) -> [(URL?, String?, Any)]? {
        return chats.elements?.map({ (chat) -> (URL?, String?, Any) in
            
            var chatImageUrl : URL? = nil
            if let imageUrl = chat.imageUrl {
                chatImageUrl = imageUrl
            }
            
            if let displayMemberImageUrl = chat.displayMemberImages?[0] {
                chatImageUrl = displayMemberImageUrl
            }
            
            return (chatImageUrl, chat.title, chat.id)
        })
    }
    
    func setMenu() {
        self.sections = [
            // MARK: - section auth start =============================================================================================
            SectionOfRow(header: "AUTH", items: [
                Row(title: "loginWithKakaoTalk()", action: {
                    if (AuthApi.isKakaoTalkLoginAvailable()) {
                        AuthApi.shared.loginWithKakaoTalk { [weak self](oauthToken, error) in
                            if let error = error {
                                self?.errorHandler(error: error)
                            }
                            else {
                                self?.success()
                                
                                let _ = oauthToken
                                //do something
                            }
                        }
                    }
                }),
                Row(title: "loginWithKakaoAccount() - iOS 11.0~", action: {
                    
                    AuthApi.shared.loginWithKakaoAccount { [weak self](oauthToken, error) in
                        if let error = error {
                            self?.errorHandler(error: error)
                        }
                        else {
                            self?.success()
                            
                            let _ = oauthToken
                             //do something
                        }
                    }
                    
                }),
                Row(title: "===== kakaotalk login(universal link) - under developing", action: {
                    print("under developing")
                })
            ]),
            // section auth end =============================================================================================
            
            // MARK: - section user start =============================================================================================
            SectionOfRow(header: "USER", items: [
                Row(title: "meForPartner()", action: {
                    UserApi.shared.meForPartner { [weak self] (user, error) in
                        if let error = error {
                            self?.errorHandler(error: error)
                        }
                        else {
                            self?.success()
                        }
                    }
                }),
                Row(title: "signup()", action: {
                    UserApi.shared.signup { [weak self] (userId, error) in
                        if let error = error {
                            self?.errorHandler(error: error)
                        }
                        else {
                            self?.success()
                        }
                    }
                }),
            ]),
            // section user end =============================================================================================
            
            // MARK: - section talk start =============================================================================================
            SectionOfRow(header: "TALK", items: [
                Row(title: "chatList()", action: {
                    TalkApi.shared.chatList { [weak self](chatList, error) in
                        if let error = error {
                            self?.errorHandler(error: error)
                        }
                        else {
                            _ = chatList
                            //do something
                            
                            self?.success()
                        }
                    }
                }),
                Row(title: "chatMembers(chatId:friendsOnly:)", action: {
                    TalkApi.shared.chatList { [weak self](chatList, error) in
                        if let error = error {
                            self?.errorHandler(error: error)
                        }
                        else {
                            self?.success()
                            
                            if let chats = chatList?.elements?.first {
                                TalkApi.shared.chatMembers(chatId: chats.id, friendsOnly: false) { [weak self] (chatMembers, error) in
                                    if let error = error {
                                        self?.errorHandler(error: error)
                                    }
                                    else {
                                        self?.success()
                                        
                                        _ = chatMembers
                                        //do something
                                    }
                                }
                            }
                            else {
                                self?.errorHandler(error:SampleError("chats empty!"))
                            }
                        }
                    }
                }),
                Row(title: "friendsForPartner()", action: {
                    TalkApi.shared.friendsForPartner { [weak self] (friends, error) in
                        if let error = error {
                            self?.errorHandler(error: error)
                        }
                        else {
                            self?.success()
                            
                            _ = friends
                            //do something
                        }
                    }
                }),
                Row(title: "friendsForPartner(context:) - recursive", action: {
                    var nextPartnerFriendContext : PartnerFriendsContext? = PartnerFriendsContext(offset: 0, limit: 3, order: .Desc)
                    
                    self.recursiveAppFriendsCompletion = { [weak self] (friends, error) -> Void in
                        guard let strongSelf = self else { return }
                                                
                        if let friends = friends  {
                            guard let next = PartnerFriendsContext(friends.afterUrl) else { return }
//                            guard let prev = PartnerFriendsContext(friends.beforeUrl) else { return }
                            nextPartnerFriendContext = next
                        }
                        
                        TalkApi.shared.friendsForPartner(context: nextPartnerFriendContext, completion: { (friends, error) in
                            if let error = error {
                                strongSelf.errorHandler(error: error)
                            }
                            else {
                                
                                strongSelf.success()                                
                                strongSelf.recursiveAppFriendsCompletion(friends, nil)
                            }
                        })
                    }

                    self.recursiveAppFriendsCompletion(nil, nil)
                }),
                
                Row(title: "friendsForPartner(context:) - PartnerFriendsContext", action: {
                    let nextPartnerFriendContext : PartnerFriendsContext? = PartnerFriendsContext(friendType:.Talk, friendFilter:.None, friendOrder:.Favorite, offset:3, limit:3, order:.Desc)
                    
                    //first
                    TalkApi.shared.friendsForPartner(context:nextPartnerFriendContext) { (friends, error) in
                        if let error = error {
                            self.errorHandler(error: error)
                        }
                        else {
                            self.success()
                            _ = friends
                        }
                    }
                }),
                Row(title: "sendDefaultMessageForPartner(templatable:receiverIdType:receiverIds:) - .Uuid", action: {
                    let title = "카카오톡 링크 4.0(메세지)"
                    let description = "친구에게보내기 티폴트 템플릿 FEED"

                    let feedTemplateJsonStringData =
                        """
                            {
                            "object_type": "feed",
                            "content": {
                            "title": "\(title)",
                            "description": "\(description)",
                            "image_url": "http://alpha-api1-kage.kakao.com/dn/cerDB5/ZSb2iRugKx/M4nuZxX823tnK1Mk5yVcv0/kakaolink40_original.png",
                            "link": {
                            "web_url": "http://dev.kakao.com",
                            "mobile_web_url": "http://dev.kakao.com"
                            }
                            },
                            "social": {
                            "like_count": 100,
                            "comment_count": 200
                            },
                            "button_title": "바로 확인"
                            }
                            """.data(using: .utf8)!
                
                            if let templatable = try? SdkJSONDecoder.custom.decode(FeedTemplate.self, from: feedTemplateJsonStringData) {
                                TalkApi.shared.friendsForPartner { [weak self] (friends, error) in
                                    if let error = error {
                                        self?.errorHandler(error: error)
                                    }
                                    else {
                                        self?.success(friends, updateIndicator: false)
                                        guard let friends = friends else {
                                            self?.errorHandler(error: error)
                                            return
                                        }
                                        
                                        if let elements = self?.extractPickerElements(friends: friends) {
                                            let vc = SDKSamplePickerViewController(title: "Friends",
                                                                                     elements: elements,
                                                                                     elementsPickerCompletionHandler: { (selectedIds) in
                                                                                        let friendIds = selectedIds as! [String]
                                                                                        
                                                                                        TalkApi.shared.sendDefaultMessageForPartner(templatable: templatable,
                                                                                                                                    receiverIdType: .Uuid,
                                                                                                                                    receiverIds: friendIds) { (messageSendResult, error) in
                                                                                            if let error = error {
                                                                                                self?.errorHandler(error: error)
                                                                                            }
                                                                                            else {
                                                                                                self?.success()
                                                                                                //do something
                                                                                                let _ = messageSendResult
                                                                                            }
                                                                                        }

                                            })
                                            self?.navigationController?.pushViewController(vc, animated: true)
                                        }
                                    }
                                }
                            }
                }),

                Row(title: "sendDefaultMessageForPartner(templatable:receiverIdType:receiverIds:) - .ChatId", action: {
                    let title = "카카오톡 링크 4.0(메세지)"
                    let description = "친구에게보내기 티폴트 템플릿 FEED"

                    let feedTemplateJsonStringData =
                        """
                            {
                            "object_type": "feed",
                            "content": {
                            "title": "\(title)",
                            "description": "\(description)",
                            "image_url": "http://alpha-api1-kage.kakao.com/dn/cerDB5/ZSb2iRugKx/M4nuZxX823tnK1Mk5yVcv0/kakaolink40_original.png",
                            "link": {
                            "web_url": "http://dev.kakao.com",
                            "mobile_web_url": "http://dev.kakao.com"
                            }
                            },
                            "social": {
                            "like_count": 100,
                            "comment_count": 200
                            },
                            "button_title": "바로 확인"
                            }
                            """.data(using: .utf8)!
//
                    if let templatable = try? SdkJSONDecoder.custom.decode(FeedTemplate.self, from: feedTemplateJsonStringData) {
                        TalkApi.shared.chatList(completion: { [weak self] (chats, error) in
                            if let error = error {
                                self?.errorHandler(error: error)
                            }
                            else {
                                self?.success(chats, updateIndicator: false)
                                guard let chats = chats else {
                                    self?.errorHandler(error: error)
                                    return
                                }
                                                                
                                if let elements = self?.extractPickerElements(chats: chats) {
                                    let vc = SDKSamplePickerViewController(title: "Chats",
                                                                         elements: elements,
                                                                         elementsPickerCompletionHandler: { (selectedIds) in
                                                                            let chatIds = selectedIds as! [Int64]
                                                                            
                                                                            TalkApi.shared.sendDefaultMessageForPartner(templatable: templatable,
                                                                                                                        receiverIdType: .ChatId,
                                                                                                                        receiverIds: chatIds) { (messageSendResult, error) in
                                                                                if let error = error {
                                                                                    self?.errorHandler(error: error)
                                                                                }
                                                                                else {
                                                                                    self?.success()
                                                                                    //do something
                                                                                    let _ = messageSendResult
                                                                                }
                                                                            }
                                    })
                                    self?.navigationController?.pushViewController(vc, animated: true)
                                }
                            }
                        })
                    }
                }),
                Row(title: "sendCustomMessageForPartner(templateId:receiverIdType:receiverIds:) - .Uuid", action: {
                    TalkApi.shared.friendsForPartner { [weak self] (friends, error) in
                        if let error = error {
                            self?.errorHandler(error: error)
                        }
                        else {
                            self?.success(friends, updateIndicator: false)
                            
                            guard let friends = friends else {
                                self?.errorHandler(error: error)
                                return
                            }
                            
                            //uuid
                            if let elements = self?.extractPickerElements(friends: friends) {
                                    let vc = SDKSamplePickerViewController(title: "Friends",
                                                                             elements: elements,
                                                                             elementsPickerCompletionHandler: { (selectedIds) in
                                        let friendIds = selectedIds as! [String]
                                        
                                        TalkApi.shared.sendCustomMessageForPartner(templateId:AppDelegate.templateId().message,
                                                                                   receiverIdType: .Uuid,
                                                                                   receiverIds: friendIds) { (messageSendResult, error) in
                                            if let error = error {
                                                self?.errorHandler(error: error)
                                            }
                                            else {
                                                self?.success()
                                                //do something
                                                let _ = messageSendResult
                                            }
                                        }
                                    })
                                    self?.navigationController?.pushViewController(vc, animated: true)
                            }
                        }
                    }
                }),
                Row(title: "sendCustomMessageForPartner(templateId:receiverIdType:receiverIds:) - .ChatId", action: {
                    TalkApi.shared.chatList(completion: { [weak self] (chats, error) in
                        if let error = error {
                            self?.errorHandler(error: error)
                        }
                        else {
                            
                            self?.success(chats, updateIndicator: false)
                            
                            guard let chats = chats else {
                                self?.errorHandler(error: error)
                                return
                            }
                                                            
                            if let elements = self?.extractPickerElements(chats: chats) {
                                let vc = SDKSamplePickerViewController(title: "Chats",
                                                                       elements: elements,
                                                                       elementsPickerCompletionHandler: { (selectedIds) in
                                            let chatIds = selectedIds as! [Int64]
                                            
                                            TalkApi.shared.sendCustomMessageForPartner(templateId:AppDelegate.templateId().message,
                                                                                       receiverIdType: .ChatId,
                                                                                       receiverIds: chatIds) { (messageSendResult, error) in
                                                if let error = error {
                                                    self?.errorHandler(error: error)
                                                }
                                                else {
                                                    self?.success()
                                                    //do something
                                                    let _ = messageSendResult
                                                }
                                            }

                                })
                                self?.navigationController?.pushViewController(vc, animated: true)
                            }
                        }
                    })
                }),
                Row(title: "sendCustomMessage(templateId:templateArgs:receiverIdType:receiverIds:) - .Uuid", action: {
                    TalkApi.shared.friendsForPartner { [weak self] (friends, error) in
                        if let error = error {
                            self?.errorHandler(error: error)
                        }
                        else {
                            self?.success(friends, updateIndicator: false)
                            
                            guard let friends = friends else {
                                self?.errorHandler(error: error)
                                return
                            }
                            
                            //uuid
                            if let elements = self?.extractPickerElements(friends: friends) {
                                    let vc = SDKSamplePickerViewController(title: "Friends",
                                                                             elements: elements,
                                                                             elementsPickerCompletionHandler: { (selectedIds) in
                                        let friendIds = selectedIds as! [String]
                                        
                                        TalkApi.shared.sendCustomMessageForPartner(templateId:AppDelegate.templateId().message,
                                                                                   templateArgs:["title":"제목입니다.", "description":"설명입니다."],
                                                                                   receiverIdType: .Uuid,
                                                                                   receiverIds: friendIds) { (messageSendResult, error) in
                                            if let error = error {
                                                self?.errorHandler(error: error)
                                            }
                                            else {
                                                self?.success()
                                                //do something
                                                let _ = messageSendResult
                                            }
                                        }
                                    })
                                    self?.navigationController?.pushViewController(vc, animated: true)
                            }
                        }
                    }
                }),
                Row(title: "sendCustomMessageForPartner(templateId:templateArgs:receiverIdType:receiverIds:) - .ChatId", action: {
                    TalkApi.shared.chatList(completion: { [weak self] (chats, error) in
                        if let error = error {
                            self?.errorHandler(error: error)
                        }
                        else {
                            
                            self?.success(chats, updateIndicator: false)
                            
                            guard let chats = chats else {
                                self?.errorHandler(error: error)
                                return
                            }
                                                            
                            if let elements = self?.extractPickerElements(chats: chats) {
                                let vc = SDKSamplePickerViewController(title: "Chats",
                                                                       elements: elements,
                                                                       elementsPickerCompletionHandler: { (selectedIds) in
                                            let chatIds = selectedIds as! [Int64]
                                            
                                            TalkApi.shared.sendCustomMessageForPartner(templateId:AppDelegate.templateId().message,
                                                                                       receiverIdType: .ChatId,
                                                                                       receiverIds: chatIds) { (messageSendResult, error) in
                                                if let error = error {
                                                    self?.errorHandler(error: error)
                                                }
                                                else {
                                                    self?.success()
                                                    //do something
                                                    let _ = messageSendResult
                                                }
                                            }

                                })
                                self?.navigationController?.pushViewController(vc, animated: true)
                            }
                        }
                    })
                }),
                Row(title: "sendScrapMessageForPartner(requestUrl:receiverUuids:) - .Uuid", action: {
                    TalkApi.shared.friendsForPartner { [weak self] (friends, error) in
                        if let error = error {
                            self?.errorHandler(error: error)
                        }
                        else {
                            self?.success(friends, updateIndicator: false)
                            
                            guard let friends = friends else {
                                self?.errorHandler(error: error)
                                return
                            }
                            
                            //uuid
                            if let elements = self?.extractPickerElements(friends: friends) {
                                    let vc = SDKSamplePickerViewController(title: "Friends",
                                                                             elements: elements,
                                                                             elementsPickerCompletionHandler: { (selectedIds) in
                                        let friendIds = selectedIds as! [String]
                                                                                
                                        TalkApi.shared.sendScrapMessageForPartner(requestUrl:"https://developers.kakao.com",
                                                                                  receiverIdType: .Uuid,
                                                                                  receiverIds: friendIds) { (messageSendResult, error) in
                                            if let error = error {
                                                self?.errorHandler(error: error)
                                            }
                                            else {
                                                self?.success()
                                                //do something
                                                let _ = messageSendResult
                                            }
                                        }
                                    })
                                    self?.navigationController?.pushViewController(vc, animated: true)
                            }
                        }
                    }
                }),
                Row(title: "sendScrapMessageForPartner(requestUrl:templateId:receiverChatIds:) - .ChatId", action: {
                    TalkApi.shared.chatList(completion: { [weak self] (chats, error) in
                        if let error = error {
                            self?.errorHandler(error: error)
                        }
                        else {
                            
                            self?.success(chats, updateIndicator: false)
                            
                            guard let chats = chats else {
                                self?.errorHandler(error: error)
                                return
                            }
                                                            
                            if let elements = self?.extractPickerElements(chats: chats) {
                                let vc = SDKSamplePickerViewController(title: "Chats",
                                                                       elements: elements,
                                                                       elementsPickerCompletionHandler: { (selectedIds) in
                                            let chatIds = selectedIds as! [Int64]
                                            
                                            TalkApi.shared.sendScrapMessageForPartner(requestUrl: "https://developers.kakao.com",
                                                                                      templateId:AppDelegate.templateId().message,
                                                                                      receiverIdType: .ChatId,
                                                                                      receiverIds: chatIds) { (messageSendResult, error) in
                                                if let error = error {
                                                    self?.errorHandler(error: error)
                                                }
                                                else {
                                                    self?.success()
                                                    //do something
                                                    let _ = messageSendResult
                                                }
                                            }

                                })
                                self?.navigationController?.pushViewController(vc, animated: true)
                            }
                        }
                    })
                }),
                
            ]),
            // section talk end =============================================================================================
            // MARK: - section link start =============================================================================================
            SectionOfRow(header: "Link", items: [
                Row(title: "-", action: {
                })
            ]),
            // section link start =============================================================================================
            
            // MARK: - section etc start =============================================================================================
            SectionOfRow(header: "ETC", items: [
                Row(title: "loginWithKakaoAccountAtAccountPage(accountParameters:)-[view:login],[email:a@kakao.com]", action: {
                    AuthApi.shared.loginWithKakaoAccount(accountParameters: ["view":"login", "email":"a@kakao.com"]) { [weak self] (token, error) in
                        if let error = error {
                            self?.errorHandler(error: error)
                        }
                        else {
                            self?.success()
                            //do something
                            
                            print("token : \(String(describing: token))")
                        }
                    }
                }),
                Row(title: "loginWithKakaoAccountAtAccountPage(accountParameters:)-[view:signup]", action: {
                    AuthApi.shared.loginWithKakaoAccount(accountParameters: ["view":"signup"]) { [weak self] (token, error) in
                        if let error = error {
                            self?.errorHandler(error: error)
                        }
                        else {
                            self?.success()
                            //do something
                            
                            print("token : \(String(describing: token))")
                        }
                    }
                }),
                Row(title: "loginWithKakaoAccountAtAccountPage(accountParameters:)-[:]", action: {
                    AuthApi.shared.loginWithKakaoAccount(accountParameters: [:]) { [weak self] (token, error) in
                        if let error = error {
                            self?.errorHandler(error: error)
                        }
                        else {
                            self?.success()
                            //do something
                            
                            print("token : \(String(describing: token))")
                        }
                    }
                })
            ])
            // section etc start =============================================================================================
        ]
    }
}
